

#use conda activate heresWaldo 
GRAPH_DATA=""
LGRAAL_OUTPUT=GRAPH_DATA+"/LGRAAL_output"
LGRAAL_LOG=GRAPH_DATA+"/LGRAAL_log"

import itertools
import os
import numpy as np

import networkx as nx

from collections import defaultdict


def main():
    pass


def format_all_timing_to_latex_table():
    graph_files = ["fly_PHY1.gw", "human_Y2H1.gw", "human_PHY1.gw", "worm_PHY1.gw", "yeast_Y2H1.gw", "yeast_PHY1.gw" ,"worm_Y2H1.gw","yeast_PHY2.gw","human_PHY2.gw","fly_Y2H1.gw"]
    all_exp_folders = os.listdir(LGRAAL_LOG)
    all_exp_folders = [f for f in all_exp_folders if ("A:" in f) and ("B:" in f)]

    indices = {k:i for i,k in enumerate(graph_files)}
    runtimes = np.zeros((len(graph_files),len(graph_files)))

    for exp in all_exp_folders:
        
        rt = get_average_runtimes(LGRAAL_LOG + "/"+exp)
        if rt != -1:

            graph_A, graph_B = exp.split("_B:")
            graph_A = graph_A.split("A:")[-1]
            i = indices[graph_A]
            j = indices[graph_B]

            runtimes[i,j] = rt
            runtimes[j,i] = rt

    return runtimes

def get_average_runtimes(log_dir):

    runtimes = []
    for log_file in os.listdir(log_dir):
        rt = parse_log_file(log_dir+"/"+log_file)
        if rt != -1:
            runtimes.append(rt)

    if len(runtimes) == 0:
        return -1
    else:
        return sum(runtimes)

def parse_log_file(file):

    with open(file,'r') as f:
        lines = f.readlines()

    if "Query graph must have less or equal number of edges " in lines[-1]:
        return -1
    else:
        return float(lines[-2].split("rt=")[-1].split("(s)")[0])
    
def print_latex_tables(data):
    n,_ = data.shape

    for i in range(n):
        row_string = "&".join([f"{x:.6f}" for x in data[i,:]]) + "\\\\"
        print(row_string)
    
def format_all_accuracy_to_latex_table():

    all_exp_folders = os.listdir(LGRAAL_OUTPUT)
    all_exp_folders = [f for f in all_exp_folders if ("A:" in f) and ("B:" in f)]

    graph_files = ["fly_PHY1.gw", "human_Y2H1.gw", "human_PHY1.gw", "worm_PHY1.gw", "yeast_Y2H1.gw", "yeast_PHY1.gw" ,"worm_Y2H1.gw","yeast_PHY2.gw","human_PHY2.gw","fly_Y2H1.gw"]
    indices = {k:i for i,k in enumerate(graph_files)} 
    opt_tri_alignments = np.zeros((len(graph_files),len(graph_files)))
    opt_edge_alignments = np.zeros((len(graph_files),len(graph_files)))

    for exp in all_exp_folders:

        if len(os.listdir(exp)) == 0: # no results, skip
            print(f"skipping experiment {exp}")
            continue
        opt_edge_res, opt_tri_ratio = find_best_alpha(exp)
        graph_A, graph_B = exp.split("_B:")
        graph_A = graph_A.split("A:")[-1]
        i = indices[graph_A]
        j = indices[graph_B]
        
        opt_edge_alignments[i,j] = opt_edge_res[1][0]
        opt_tri_alignments[i,j] = opt_tri_ratio[1][1]

        print(f"finished {exp} with best edge result {opt_edge_alignments[i,j]}, best tri results {opt_tri_alignments[i,j]}")
        
        opt_edge_alignments[j,i] = opt_edge_alignments[i,j]
        opt_tri_alignments[j,i] = opt_tri_alignments[i,j]

    return opt_edge_alignments, opt_tri_alignments

#gets the choice of alpha which has the best 
def find_best_alpha(exp_folder):

   matchings = [f for f in os.listdir(exp_folder) if ".matching" in f]
   results = [(f,eval_matching(exp_folder + "/"+ f)) for f in matchings]

   #return max edge ratio, max tri ratio
   return max(results,key=lambda x:x[1][0]), max(results,key=lambda x:x[1][1]),  
    

def eval_matching(matching_file):

    matching = read_matching(matching_file)
    
    #get the relavant graph names    
#    graph_A, graph_B = matching_file.split("/")[-2].split("_B:") #graph folder is named with files
    graph_A, graph_B = matching_file.split("/")[-1].split("_alpha")[0].split("_B:") 

    ext = ".gw"
    graph_A = graph_A.split("A:")[-1] + ext
    graph_B = graph_B + ext
    
    A = nx.read_leda(GRAPH_DATA +'/' + graph_A)
    B = nx.read_leda(GRAPH_DATA +'/' + graph_B)

#    return A,B, matching
#    x = count_matched_edges(A,B,matching)
    EdgesMatched, maxEdges = count_matched_edges(A,B,matching)
    TrisMatched, maxTris = count_matched_triangles(A,B,matching) 

#    return A_tris, B_tris
    return EdgesMatched/maxEdges, TrisMatched/maxTris 
 #   return EdgesMatched, TrisMatched

def count_matched_edges(A,B,m):
    """
        Given a matching m between two networks A and B, the function returns 
      the number of edges matched and the maximum possible edges to match between
       the two.  
    """
    
    #use the smaller of the two networkx for evaluation
    if len(A.edges) < len(B.edges):
        edges = A.edges
        other = B.edges
    else:
        edges = B.edges
        other = A.edges

    aligned_edges = 0
    for (v_i,v_j) in edges:
        if other.get((m[v_i],m[v_j]),-1) != -1:
          aligned_edges += 1

    return aligned_edges, min(len(A.edges),len(B.edges))




def count_matched_triangles(A,B,m):
    """
        Given a matching m between two networks A and B, the function returns 
      the number of matched triangles and the maximum possible triangles to 
      match between the two.  
    """

    #iteration found @ https://groups.google.com/forum/#!topic/networkx-discuss/baNlK-DU_B0
    A_tris = [frozenset(triple) for triple in set(frozenset((n,nbr,nbr2)) for n in A for nbr, nbr2 in itertools.combinations(A[n],2) if nbr in A[nbr2])]

    B_tris = [frozenset(triple) for triple in set(frozenset((n,nbr,nbr2)) for n in B for nbr, nbr2 in itertools.combinations(B[n],2) if nbr in B[nbr2])]

    
    if len(A_tris) < len(B_tris):
        iter_tris = A_tris
        check_tris = set(B_tris)
    else:
        iter_tris = B_tris
        check_tris = set(A_tris)

#    return A_tris,B_tris

    matched_triangles = 0
    for (v_i,v_j,v_k) in iter_tris:
#      print([m[v_i],m[v_j],m[v_k]])
      if frozenset([m[v_i],m[v_j],m[v_k]]) in check_tris:
        matched_triangles += 1

    return matched_triangles, min(len(A_tris),len(B_tris))
        
    

#produces the matching from graph A to graph B
def read_matching(matching_file):

    matching_dict = defaultdict(lambda : -1)
    with open(matching_file,'r') as f:
        for line in f:
            if line == "'\t\n'":
                continue
            else:
                v_i, v_j = line.split('\t')
                v_j = v_j.strip()

                matching_dict[v_i] = v_j

    return matching_dict


if __name__=="__main__":
    main()
